@@ -75,6 +75,7 @@ Central place to keep many in-game text strings
 #define GAMETEXT_CHANGEKEY            _("Change Key...")
 #define GAMETEXT_CHATBOXHELP          _("Use the tabulation key to complete player names. Use the * to send private messages.")
 #define GAMETEXT_CHATDIALOG           _("Open chat dialog box")
+#define GAMETEXT_CHATPRIVATEDIALOG    _("Open private chat dialog box")
 #define GAMETEXT_CHECKINGFORLEVELS    _("Checking for new or updated levels...")
 #define GAMETEXT_CHILDREN_COMPLIANT   _("I'm a child")
 #define GAMETEXT_CHOOSELEVEL          _("Choose Level")
@@ -358,6 +359,10 @@ std::string(_("You control your bike using the keyboard")) + ":\n" \
 #define GAMETEXT_LEVELINFO            _("Level Info...")
 #define GAMETEXT_LEVELNAME            _("Level Name")
 #define GAMETEXT_LEVELCANNOTBELOADED  _("Level '%s' cannot be loaded")
+#define GAMETEXT_LEVELLOADINGFAILURE _("The level failed while loading with the following error :")
+#define GAMETEXT_SENDMSGTOADMINS     _("Do you want to send a report to website administrators ?")
+#define GAMETEXT_SENDING_REPORT      _("Sending the report...")
+#define GAMETEXT_SENDING_VOTE        _("Sending the vote...")
 #define GAMETEXT_LEVELPACK            _("Level Pack")
 #define GAMETEXT_LEVELPACKS           _("Level Packs")
 #define GAMETEXT_LEVELREQUIREDBYREPLAY _("Level '%s' required by replay!")
@@ -505,6 +510,8 @@ std::string(_("You control your bike using the keyboard")) + ":\n" \
 #define GAMETEXT_SWITCHFAVORITE       _("Add/remove to favorite levels")
 #define GAMETEXT_SWITCHFPS            _("Switch FPS")
 #define GAMETEXT_SWITCHBLACKLIST      _("Add/remove to blacklisted levels")
+#define GAMETEXT_SWITCHNETMODE        _("Switch net game mode")
+#define GAMETEXT_SWITCHHIGHSCOREINFORMATION _("Switch highscore information")
 #define GAMETEXT_SWITCHWWWACCESS      _("Switch WWW access")
 #define GAMETEXT_SWITCHGFXQUALITYMODE _("Switch graphics quality mode")
 #define GAMETEXT_SWITCHGFXMODE        _("Switch graphics mode")
@@ -915,6 +922,7 @@ std::string(_("You control your bike using the keyboard")) + ":\n" \
 #define SYS_MSG_RESTART_LEVEL           _("Restart Level to activate!")
 
 #define SYS_MSG_NO_NEXT_HIGHSCORE(A)    ngettext("Can't find a highscore in the next %i level", "Can't find a highscore in the next %i levels", A)
+#define SYS_MSG_UNABLE_TO_BUILD_THE_REPORT _("Unable to build the report")
 
 // font to use to choice the GROUP : currently, two groups, GENERAL and ASIAN
 #define FONT_GROUP _("FontGroup:GENERAL")
