@@ -269,6 +269,8 @@ InputEventType InputHandler::joystickAxisSens(Sint16 m_joyAxisValue) {
       IFullKey("KeyRestartCheckpoint",   XMKey(SDLK_BACKSPACE, KMOD_NONE),  GAMETEXT_RESTARTCHECKPOINT);
     m_globalKeys[INPUT_CHAT]                =
       IFullKey("KeyChat",                XMKey(SDLK_c,         KMOD_LCTRL), GAMETEXT_CHATDIALOG);
+    m_globalKeys[INPUT_CHATPRIVATE]         =
+      IFullKey("KeyChatPrivate",                XMKey(SDLK_p,         KMOD_LCTRL), GAMETEXT_CHATPRIVATEDIALOG);
     m_globalKeys[INPUT_LEVELWATCHING]          =
       IFullKey("KeyLevelWatching",       XMKey(SDLK_TAB,       KMOD_NONE),  GAMETEXT_LEVELWATCHING);
     m_globalKeys[INPUT_SWITCHPLAYER]           =
@@ -291,6 +293,10 @@ InputEventType InputHandler::joystickAxisSens(Sint16 m_joyAxisValue) {
       IFullKey("KeySwitchGFXQualityMode", XMKey(SDLK_F10, KMOD_NONE), GAMETEXT_SWITCHGFXQUALITYMODE);
     m_globalKeys[INPUT_SWITCHGFXMODE] =
       IFullKey("KeySwitchGFXMode", XMKey(SDLK_F11, KMOD_NONE), GAMETEXT_SWITCHGFXMODE);
+    m_globalKeys[INPUT_SWITCHNETMODE] =
+      IFullKey("KeySwitchNetMode", XMKey(SDLK_n, KMOD_LCTRL), GAMETEXT_SWITCHNETMODE);
+    m_globalKeys[INPUT_SWITCHHIGHSCOREINFORMATION] =
+      IFullKey("KeySwitchHighscoreInformation", XMKey(SDLK_w, KMOD_LCTRL), GAMETEXT_SWITCHHIGHSCOREINFORMATION);
 
     // uncustomizable keys
     m_globalKeys[INPUT_HELP] =
