@@ -63,8 +63,11 @@ enum INPUT_GLOBALKEYS {
   INPUT_SWITCHGFXMODE,
   INPUT_SHOWCONSOLE,
   INPUT_CHAT,
+  INPUT_CHATPRIVATE,
   INPUT_CONSOLEHISTORYPLUS,
   INPUT_CONSOLEHISTORYMINUS,
+  INPUT_SWITCHNETMODE,
+  INPUT_SWITCHHIGHSCOREINFORMATION,
 
   // uncustomizable keys
   INPUT_HELP,
