@@ -20,15 +20,15 @@ Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
 #include "LuaLibGame.h"
 #include "helpers/VExcept.h"
-#include "xmscene/Scene.h"
 #include "GameEvents.h"
 #include "Input.h"
 #include "Locales.h"
+#include "xmscene/Scene.h"
 #include "helpers/Log.h"
 #include "xmscene/Level.h"
 #include "xmscene/Block.h"
 #include "xmscene/ScriptTimer.h"
-#include <sstream>
+#include "XMSession.h"
 
 #define STIMER_DELAY_DEFAULT 100
 #define STIMER_LOOPS_DEFAULT 0
@@ -54,6 +54,7 @@ luaL_reg LuaLibGame::m_gameFuncs[] = {
   {"SetEntityPos",                 LuaLibGame::L_Game_SetEntityPos},
   {"SetKeyHook",                   LuaLibGame::L_Game_SetKeyHook},
   {"GetKeyByAction",               LuaLibGame::L_Game_GetKeyByAction},
+  {"GetKeyByActionTech",           LuaLibGame::L_Game_GetKeyByActionTech},
   {"Log",                          LuaLibGame::L_Game_Log},
   {"SetBlockCenter",               LuaLibGame::L_Game_SetBlockCenter},
   {"SetBlockRotation",             LuaLibGame::L_Game_SetBlockRotation},
@@ -91,209 +92,30 @@ luaL_reg LuaLibGame::m_gameFuncs[] = {
   {"GetPlayerVelocity",            LuaLibGame::L_Game_GetPlayerVelocity},
   {"GetPlayerSpeed",               LuaLibGame::L_Game_GetPlayerSpeed},
   {"GetPlayerAngle",               LuaLibGame::L_Game_GetPlayerAngle}, 
-  {"StartTimer",               LuaLibGame::L_Game_StartTimer},   
-  {"SetTimerDelay",               LuaLibGame::L_Game_SetTimerDelay}, 
-  {"StopTimer",               LuaLibGame::L_Game_StopTimer}, 
-	{"SetCameraPosition",               LuaLibGame::L_Game_SetCameraPosition}, 
+  {"GetPlayerProfileName",         LuaLibGame::L_Game_GetPlayerProfileName},
+  {"StartTimer",                   LuaLibGame::L_Game_StartTimer},   
+  {"SetTimerDelay",                LuaLibGame::L_Game_SetTimerDelay}, 
+  {"StopTimer",                    LuaLibGame::L_Game_StopTimer}, 
+  {"SetCameraPosition",            LuaLibGame::L_Game_SetCameraPosition}, 
   {NULL, NULL}
 };
-Scene*     LuaLibGame::m_exec_world              = NULL;
+
+Scene*        LuaLibGame::m_exec_world              = NULL;
 InputHandler* LuaLibGame::m_exec_activeInputHandler = NULL;
 
-/*===========================================================================
-  Lua 5.1 compatibility code (Following is from lua 5.0.2)
-  ===========================================================================*/
-static void X_tag_error (lua_State *L, int narg, int tag) {
-  luaL_typerror(L, narg, lua_typename(L, tag)); 
-}
-
-lua_Number X_luaL_check_number(lua_State *L,int narg) {
-  lua_Number d = lua_tonumber(L, narg);
-  if (d == 0 && !lua_isnumber(L, narg))  /* avoid extra test when d is not 0 */
-    X_tag_error(L, narg, LUA_TNUMBER);
-  return d;    
-}
-
-LuaLibGame::LuaLibGame(Scene *i_pScene) {
-  m_pL = lua_open();
-  luaopen_base(m_pL);   
-  luaopen_math(m_pL);
-  luaopen_table(m_pL);
-  luaL_openlib(m_pL, "Game", m_gameFuncs, 0);
-
-  m_pScene           = i_pScene;
+LuaLibGame::LuaLibGame(Scene *i_pScene) : LuaLibBase("Game", m_gameFuncs) {
+  m_pScene              = i_pScene;
   m_pActiveInputHandler = InputHandler::instance();
 }
 
 LuaLibGame::~LuaLibGame() {
-  lua_close(m_pL);
 }
 
-void LuaLibGame::setWorld() {
+void LuaLibGame::setInstance() {
   m_exec_world              = m_pScene;
   m_exec_activeInputHandler = m_pActiveInputHandler;
 }
 
-/*===========================================================================
-  Simple lua interaction
-  ===========================================================================*/
-bool LuaLibGame::scriptCallBool(const std::string& FuncName,bool bDefault) {
-  setWorld();
-  
-  bool bRet = bDefault;
-  
-  /* Fetch global function */
-  lua_getglobal(m_pL,FuncName.c_str());
-  
-  /* Is it really a function and not just a pile of ****? */
-  if(lua_isfunction(m_pL,-1)) {
-    /* Call! */
-    if(lua_pcall(m_pL,0,1,0) != 0) {
-      throw Exception("failed to invoke (bool) " + FuncName + std::string("(): ") + std::string(lua_tostring(m_pL,-1)));
-    }
-    
-    /* Retrieve return value */
-    if(!lua_toboolean(m_pL,-1)) {      
-      bRet = false;
-    }
-    else {
-      bRet = true;
-    }
-  }
-  
-  /* Reset Lua VM */
-  lua_settop(m_pL,0);        
-  
-  return bRet;
-}
-
-void LuaLibGame::scriptCallVoid(const std::string& FuncName) {
-  setWorld();
-  
-  /* Fetch global function */
-  lua_getglobal(m_pL,FuncName.c_str());
-  
-  /* Is it really a function and not just a pile of ****? */
-  if(lua_isfunction(m_pL,-1)) {
-    /* Call! */
-    if(lua_pcall(m_pL,0,0,0) != 0) {
-      throw Exception("failed to invoke (void) " + FuncName + std::string("(): ") + std::string(lua_tostring(m_pL,-1)));
-    }      
-  }
-  
-  /* Reset Lua VM */
-  lua_settop(m_pL,0);        
-}
-
-void LuaLibGame::scriptCallVoidNumberArg(const std::string& FuncName, int n) {
-  setWorld();
-  
-  /* Fetch global function */
-  lua_getglobal(m_pL,FuncName.c_str());
-  
-  /* Is it really a function and not just a pile of ****? */
-  if(lua_isfunction(m_pL,-1)) {
-    /* Call! */
-    lua_pushnumber(m_pL, n);
-    if(lua_pcall(m_pL,1,0,0) != 0) {
-      throw Exception("failed to invoke (void) " + FuncName + std::string("(): ") + std::string(lua_tostring(m_pL,-1)));
-    }      
-  }
-  
-  /* Reset Lua VM */
-  lua_settop(m_pL,0);        
-}
-
-void LuaLibGame::scriptCallVoidNumberArg(const std::string& FuncName, int n1, int n2) {
-  setWorld();
-  
-  /* Fetch global function */
-  lua_getglobal(m_pL,FuncName.c_str());
-  
-  /* Is it really a function and not just a pile of ****? */
-  if(lua_isfunction(m_pL,-1)) {
-    /* Call! */
-    lua_pushnumber(m_pL, n1);
-    lua_pushnumber(m_pL, n2);
-    if(lua_pcall(m_pL,2,0,0) != 0) {
-      throw Exception("failed to invoke (void) " + FuncName + std::string("(): ") + std::string(lua_tostring(m_pL,-1)));
-    }      
-  }
-  
-  /* Reset Lua VM */
-  lua_settop(m_pL,0);        
-}
-
-void LuaLibGame::scriptCallTblVoid(const std::string& Table, const std::string& FuncName) {
-  setWorld();
-  
-  /* Fetch global table */        
-  lua_getglobal(m_pL,Table.c_str());
-  
-  //    printf("[%s.%s]\n",Table.c_str(),FuncName.c_str());
-  
-  if(lua_istable(m_pL,-1)) {
-    lua_pushstring(m_pL,FuncName.c_str());
-    lua_gettable(m_pL,-2);
-    
-    if(lua_isfunction(m_pL,-1)) {
-      /* Call! */
-      if(lua_pcall(m_pL,0,0,0) != 0) {
-	throw Exception("failed to invoke (tbl,void) " + Table + std::string(".") + 
-			FuncName + std::string("(): ") + std::string(lua_tostring(m_pL,-1)));
-      }              
-    }
-  }
-  
-  /* Reset Lua VM */
-  lua_settop(m_pL,0);        
-}
-
-void LuaLibGame::scriptCallTblVoid(const std::string& Table, const std::string& FuncName, int n) {
-  setWorld();
-  
-  /* Fetch global table */        
-  lua_getglobal(m_pL,Table.c_str());
-  
-  //    printf("[%s.%s]\n",Table.c_str(),FuncName.c_str());
-  
-  if(lua_istable(m_pL,-1)) {
-    lua_pushstring(m_pL,FuncName.c_str());
-    lua_gettable(m_pL,-2);
-    
-    if(lua_isfunction(m_pL,-1)) {
-      /* Call! */
-      lua_pushnumber(m_pL, n);
-      if(lua_pcall(m_pL,1,0,0) != 0) {
-	throw Exception("failed to invoke (tbl,void) " + Table + std::string(".") + 
-			FuncName + std::string("(): ") + std::string(lua_tostring(m_pL,-1)));
-      }              
-    }
-  }
-  
-  /* Reset Lua VM */
-  lua_settop(m_pL,0);        
-}
-
-void LuaLibGame::loadScript(const std::string& i_scriptCode, const std::string& i_scriptFilename) {
-  /* Use the Lua aux lib to load the buffer */
-  int nRet;
-
-  setWorld();
-
-  nRet = luaL_loadbuffer(m_pL, i_scriptCode.c_str(), i_scriptCode.length(),
-			 i_scriptFilename.c_str()) || lua_pcall(m_pL, 0, 0, 0);    
-
-  /* Returned WHAT? */
-  if(nRet != 0) {
-    throw Exception("failed to load level script");
-  }
-}
-
-std::string LuaLibGame::getErrorMsg() {
-  return lua_tostring(m_pL,-1);
-}
-
 /*===========================================================================
   Game.* 
   Lua game library functions
@@ -358,9 +180,14 @@ int LuaLibGame::L_Game_Message(lua_State *pL) {
 
   /* Convert all arguments to strings */
   std::string Out;
-  for(int i=0;i<lua_gettop(pL);i++) 
-    Out.append(_(luaL_checkstring(pL, i+1)));
-  
+  std::string Temp;
+  for(int i=0;i<lua_gettop(pL);i++) {
+    Temp = luaL_checkstring(pL, i+1);
+    if(Temp != "") {
+      Out.append(_(Temp.c_str()));
+    }
+  }  
+
   m_exec_world->createGameEvent(new MGE_Message(m_exec_world->getTime(), Out));
   return 0;
 }  
@@ -536,8 +363,11 @@ int LuaLibGame::L_Game_SetKeyHook(lua_State *pL) {
   if(m_exec_activeInputHandler != NULL) {
     try {
       m_exec_activeInputHandler->addScriptKeyHook(m_exec_world, luaL_checkstring(pL,1), luaL_checkstring(pL,2));
-    } catch(Exception &e) {  
-			luaL_error (pL, "AddScriptKeyHook failed");    
+    } catch(Exception &e) {
+      std::string v_key  = luaL_checkstring(pL,1);
+      std::string v_func = luaL_checkstring(pL,2);
+      std::string v_msg = "SetKeyHook(key=" + v_key + ", function=" + v_func + ") failed\n" + e.getMsg();
+	luaL_error (pL, v_msg.c_str());
     }
   }
   return 0;
@@ -547,7 +377,17 @@ int LuaLibGame::L_Game_GetKeyByAction(lua_State *pL) {
   /* no event for this */
 
   if(m_exec_activeInputHandler != NULL) {
-    lua_pushstring(pL,m_exec_activeInputHandler->getFancyKeyByAction(luaL_checkstring(pL,1)).c_str());
+    lua_pushstring(pL,m_exec_activeInputHandler->getKeyByAction(luaL_checkstring(pL,1)).c_str());
+    return 1;
+  }
+  return 0;
+}  
+
+int LuaLibGame::L_Game_GetKeyByActionTech(lua_State *pL) {
+  /* no event for this */
+
+  if(m_exec_activeInputHandler != NULL) {
+    lua_pushstring(pL,m_exec_activeInputHandler->getKeyByAction(luaL_checkstring(pL,1), true).c_str());
     return 1;
   }
   return 0;
@@ -704,7 +544,7 @@ int LuaLibGame::L_Game_KillPlayer(lua_State *pL) {
 }
 
 int LuaLibGame::L_Game_KillEntity(lua_State *pL) {
-  m_exec_world->createKillEntityEvent(luaL_checkstring(pL,1));
+  m_exec_world->createExternalKillEntityEvent(luaL_checkstring(pL,1));
   return 0;
 }
 
@@ -867,7 +707,7 @@ int LuaLibGame::L_Game_StopMusic(lua_State *pL) {
 int LuaLibGame::L_Game_GetPlayerVelocity(lua_State *pL) {
   /* no event for this */
   int v_player = (int)X_luaL_check_number(pL,1);
-
+  
   if(v_player < 0 || (unsigned int)v_player >= m_exec_world->Players().size()) {
 		luaL_error (pL, "Invalid player number");
   }
@@ -900,6 +740,17 @@ int LuaLibGame::L_Game_GetPlayerAngle(lua_State *pL) {
   return 1; //return 1 value
 }
 
+int LuaLibGame::L_Game_GetPlayerProfileName(lua_State *pl) {
+  /* no event for this */
+  std::string v_profileName;
+  // deprecated cause people used a test on it to make the level harder
+  //v_profileName = XMSession::instance()->profile();
+  v_profileName = "*deprecated*";
+  lua_pushstring(pl,v_profileName.c_str());
+  
+  return 1;
+}
+
 /*=====================================================
         Script Timer Functions! by tuhoojabotti
 =====================================================*/
@@ -962,24 +813,3 @@ int LuaLibGame::L_Game_StopTimer(lua_State *pL) {
 	}
   return 0; //return no values to the script
 }
-/*=====================================================
-            Script Helpers
-=====================================================*/
-
-int LuaLibGame::args_numberOfArguments(lua_State *pL) {
-  return lua_gettop(pL);
-}
-
-void LuaLibGame::args_CheckNumberOfArguments(lua_State *pL, int i_from, int i_to) {
-  int n = args_numberOfArguments(pL);
-
-  if(i_to == -1) {
-    if(n != i_from) {
-      luaL_error (pL, "Invalid number of arguments");  
-    }
-  } else {
-    if(n < i_from || n > i_to) {
-      luaL_error (pL, "Invalid number of arguments");  
-    }
-  }
-}
