@@ -28,12 +28,13 @@ class DrawLib;
 class FontManager;
 class Sprite;
 class GameApp;
+class RenderSurface;
 
 #include "../../Theme.h"
 #include "../../include/xm_SDL.h"
 
 #define GUI_JOYSTICK_MINIMUM_DETECTION 3000
-
+#define UGLY_MODE_WINDOW_BG MAKE_COLOR(35,35,35,255)
 
 /*===========================================================================
   Alignments
@@ -132,7 +133,9 @@ enum UIMsgBoxButton {
   UI_MSGBOX_CANCEL      = 2,
   UI_MSGBOX_YES         = 4,
   UI_MSGBOX_NO          = 8,
-  UI_MSGBOX_YES_FOR_ALL = 16
+  UI_MSGBOX_YES_FOR_ALL = 16,
+  UI_MSGBOX_CUSTOM1     = 32,
+  UI_MSGBOX_CUSTOM2     = 64
 };
   
 class UIWindow {
@@ -206,9 +209,15 @@ public:
       
   UIWindow *getChild(std::string Child);
   UIRoot *getRoot(void);
+  RenderSurface* getScreen();
       
   /* Generic message boxing */
-  UIMsgBox *msgBox(std::string Text,UIMsgBoxButton Buttons,bool bTextInput=false,bool bQuery=false,bool i_verticallyLarge=false);
+  UIMsgBox *msgBox(std::string Text,UIMsgBoxButton Buttons, const std::string& i_help,
+		   const std::string& i_custom1 = "", const std::string& i_custom2 = "",
+		   bool bTextInput=false,bool bQuery=false,bool i_verticallyLarge=false);
+  UIMsgBox *msgBox(std::string Text, std::vector<std::string>& wordcompletionlist,UIMsgBoxButton Buttons, const std::string& i_help,
+		   const std::string& i_custom1 = "", const std::string& i_custom2 = "",
+		   bool bTextInput=false,bool bQuery=false,bool i_verticallyLarge=false);
     
   /* Data interface */
   UIWindow *getPrimaryChild(void) {return m_pPrimaryChild;}
@@ -258,6 +267,9 @@ protected:
       
   static DrawLib* m_drawLib;
     
+  // screen to draw ; the same as the parent, but copy to not get in cascade
+  RenderSurface* m_screen;
+
 private:
   /* Data */
   std::string m_ContextHelp;                /* Context help */
@@ -265,6 +277,7 @@ private:
 					       the window transists from "hidden" to "shown". 
 					       useful for specifying "default" buttons in menus */
   UIWindow *m_pParent;                      /* Parent window */
+
   std::string m_ID;                         /* Non-unique id */
   std::vector<UIWindow *> m_Children;       /* Child windows */  
   UIRect m_Pos;                             /* Position */
@@ -400,14 +413,8 @@ private:
 class UIMsgBox : public UIFrame {
 public:
   UIMsgBox() {m_bTextInput = false;m_nNumButtons=0;}
-  UIMsgBox(UIWindow *pParent,int x=0,int y=0,std::string Caption="",int nWidth=0,int nHeight=0) {
-    initW(pParent,x,y,Caption,nWidth,nHeight);
-
-    setStyle(UI_FRAMESTYLE_TRANS);
-    m_textInputFont = NULL;
-    m_bTextInput = false;
-    m_nNumButtons = 0;
-  }      
+  UIMsgBox(UIWindow *pParent,int x=0,int y=0,std::string Caption="",int nWidth=0,int nHeight=0);
+  UIMsgBox(UIWindow *pParent, std::vector<std::string>& wordcompletionlist,int x=0,int y=0,std::string Caption="",int nWidth=0,int nHeight=0);
   virtual ~UIMsgBox() {_ReEnableSiblings();}
     
   /* Virtual methods */
@@ -419,10 +426,18 @@ public:
   bool setClicked(std::string Text);
   UIMsgBoxButton getClicked(void);
   void enableTextInput(void) {m_bTextInput=true;}
-  std::string getTextInput(void) {return m_TextInput;}
-  void setTextInput(std::string s) {m_TextInput=s;}
+  std::string getTextInput(void) {return m_TextInput_real;}
+  void setTextInput(std::string s) {m_TextInput_real=s;m_TextInput_fake=s;}
   void setTextInputFont(FontManager* pFont) {m_textInputFont = pFont;}
+  void addCompletionWord(std::string& word);
+  void addCompletionWord(std::vector<std::string>& list);
+
+  std::string getCustom1() const;
+  void setCustom1(const std::string& i_str);
       
+  std::string getCustom2() const;
+  void setCustom2(const std::string& i_str);
+
   /* Data interface */
   void addButton(UIButton *p) {m_pButtons[m_nNumButtons++] = p;}
   std::vector<bool> &getSiblingStates(void) {return m_SiblingStates;}
@@ -430,18 +445,26 @@ public:
   void makeActiveButton(UIMsgBoxButton i_button);
       
 private:
+  void initMsgBox(UIWindow *pParent,int x,int y,std::string Caption,int nWidth,int nHeight);
+
   /* Data */
   std::vector<bool> m_SiblingStates;
   UIMsgBoxButton m_Clicked;
   UIButton *m_pButtons[10];
   unsigned int m_nNumButtons;
+
+  std::string m_custom1, m_custom2;
       
   bool m_bTextInput;
-  std::string m_TextInput;
+  std::string m_TextInput_real;
+  std::string m_TextInput_fake;
   FontManager* m_textInputFont;
+  std::vector<std::string> m_completionWords;
       
   /* Helpers */
   void _ReEnableSiblings(void);
+  std::vector<std::string> findMatches();
+  void showMatch();
 };
 
 /*===========================================================================
@@ -824,7 +847,8 @@ struct UIRootActCandidate {
   
 class UIRoot : public UIWindow {
 public:
-  UIRoot();
+  UIRoot(RenderSurface* i_screen);
+  ~UIRoot();
     
   /* Methods */
   virtual void paint(void);            
