@@ -22,6 +22,7 @@ Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 #include "../../helpers/Log.h"
 #include "../../helpers/VExcept.h"
 #include "../../helpers/utf8.h"
+#include "../../helpers/System.h"
 #include <string>
 #include <sstream>
 #include "../helpers/Net.h"
@@ -36,6 +37,7 @@ Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 #include "../../db/xmDatabase.h"
 #include "../../xmscene/Level.h"
 #include "../../xmscene/BikeController.h"
+#include "../ServerRules.h"
 
 #define XM_SERVER_SLAVE_MODE_MIN_PROTOCOL_VERSION 1
 
@@ -48,13 +50,15 @@ Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 #define XM_SERVER_PREPLAYING_TIME 300
 #define XM_SERVER_DEFAULT_BAN_NBDAYS 30
 #define XM_SERVER_MAX_FOLLOWING_UDP 100
-#define XM_SERVER_MIN_INACTIVITY_LOOP_TO_SLEEP 1000
 #define XM_SERVER_DEFAULT_BANNER "Welcome on this server"
 #define XM_SERVER_UNPLAYING_SLEEP 10
+#define XM_SERVER_NICK_LENGTH_MAX 16
 
 // limit multi private message to avoid people spamming everybody and making think it's private
 #define XM_SERVER_MAXIMUM_MULTI_PRIVATE_MESSAGE 3
 
+#define XM_SERVER_DEFAULT_RULES "Rules/classical.rules"
+
 NetSClient::NetSClient(unsigned int i_id, TCPsocket i_tcpSocket, IPaddress *i_tcpRemoteIP) {
     m_id   = i_id;
     m_mode = NETCLIENT_GHOST_MODE;
@@ -72,6 +76,13 @@ NetSClient::NetSClient(unsigned int i_id, TCPsocket i_tcpSocket, IPaddress *i_tc
     m_isAdminConnected = false;
     m_lastGhostFrameTime = 0;
     m_protocolVersion = -1; // not set
+    m_udpRemoteIP.host = INADDR_NONE;
+    m_udpRemoteIP.port = 0;
+
+    // last ping information
+    m_lastPing.id = -1;
+    m_lastPing.pingTime = -1;
+    m_lastPing.pongTime = -1;
 }
 
 NetSClient::~NetSClient() {
@@ -94,6 +105,10 @@ int NetSClient::points() {
   return m_points;
 }
 
+void NetSClient::setPoints(int i_points) {
+  m_points = i_points;
+}
+
 void NetSClient::addPoints(int i_points) {
   m_points += i_points;
 }
@@ -198,7 +213,22 @@ void NetSClient::setLastInactivTimeAlert(int i_time) {
   m_lastInactivTimeAlert = i_time;
 }
 
-void NetSClient::markToPlay(bool i_value) {
+void NetSClient::markToPlay(ServerRules* i_rules, bool i_value) {
+
+  // do a rule action only if the value changes
+  if(i_value != m_isMarkedToPlay) {
+    try {
+      if(i_value) {
+	i_rules->scriptCallVoidNumberArg("Global_whenPlayer_added",   id());
+      } else {
+	i_rules->scriptCallVoidNumberArg("Global_whenPlayer_removed", id());
+      }
+    } catch(Exception &e) {
+      // continue the game even if the rules are badly written
+      LogError(std::string("Rules: " + e.getMsg()).c_str());
+    }
+  }
+
   m_isMarkedToPlay = i_value;
 }
 
@@ -228,8 +258,15 @@ std::string NetSClient::playingLevelId() const {
   return m_playingLevelId;
 }
 
-ServerThread::ServerThread(const std::string& i_dbKey) 
+NetPing* NetSClient::lastPing() {
+  return &m_lastPing;
+}
+
+ServerThread::ServerThread(const std::string& i_dbKey, int i_port, const std::string& i_adminPassword) 
   : XMThread(i_dbKey) {
+    m_port = i_port;
+    m_adminPassword = i_adminPassword;
+
     m_set = NULL;
     m_nextClientId = 0;
     m_udpPacket = SDLNet_AllocPacket(XM_SERVER_MAX_UDP_PACKET_SIZE);
@@ -237,15 +274,18 @@ ServerThread::ServerThread(const std::string& i_dbKey)
     m_universe = NULL;
     m_DBuffer = new DBuffer();
     m_DBuffer->initOutput(XM_NET_MAX_EVENTS_SHOT_SIZE);
-    SP2_setPhase(SP2_PHASE_WAIT_CLIENTS);
+    m_sp2phase           = SP2_PHASE_NONE;
     m_lastFrameTimeStamp = -1;
     m_frameLate          = 0;
     m_currentFrame       = 0;
     m_nFollowingUdp      = 0;
-    m_nInactivNetLoop    = 0;
     m_startTimeStr       = GameApp::getTimeStamp();
     m_banner             = XM_SERVER_DEFAULT_BANNER;
     m_acceptConnections  = false;
+    m_unmanagedActions   = 0;
+    m_rules = NULL;
+    m_needToReloadRules  = false;
+    m_sceneHook = new XMServerSceneHooks(this);
 
     if(!m_udpPacket) {
       throw Exception("SDLNet_AllocPacket: " + std::string(SDLNet_GetError()));
@@ -255,17 +295,31 @@ ServerThread::ServerThread(const std::string& i_dbKey)
 ServerThread::~ServerThread() {
   SDLNet_FreePacket(m_udpPacket);
   delete m_DBuffer;
+  if(m_rules != NULL) {
+    delete m_rules;
+  }
+  delete m_sceneHook;
 }
 
 int ServerThread::realThreadFunction() {
   IPaddress ip;
   int ssn;
-  unsigned int i;
 
   LogInfo("server: starting");
 
+  // this is not really required here, but it will not start the server if an error occured
+  // init rules
+  try {
+    reloadRules(XM_SERVER_DEFAULT_RULES);
+  } catch(Exception &e) {
+    LogError((e.getMsg() + "\n" + m_rules->getErrorMsg()).c_str());
+    return 1;
+  }
+
+  LogInfo("server: ports %i UDP & TCP", m_port);
+
   /* Resolving the host using NULL make network interface to listen */
-  if(SDLNet_ResolveHost(&ip, NULL, XMSession::instance()->serverPort()) < 0) {
+  if(SDLNet_ResolveHost(&ip, NULL, m_port) < 0) {
     LogError("server: SDLNet_ResolveHost: %s", SDLNet_GetError());
     return 1;
   }
@@ -284,7 +338,7 @@ int ServerThread::realThreadFunction() {
     return 1;
   }
 
-  if((m_udpsd = SDLNet_UDP_Open(XMSession::instance()->serverPort())) == 0) {
+  if((m_udpsd = SDLNet_UDP_Open(m_port)) == 0) {
     LogError("server: SDLNet_UDP_Open: %s", SDLNet_GetError());
     SDLNet_FreeSocketSet(m_set);
     SDLNet_TCP_Close(m_tcpsd);
@@ -313,6 +367,9 @@ int ServerThread::realThreadFunction() {
     StateManager::instance()->sendAsynchronousMessage("SERVER_STATUS_CHANGED");
   }
 
+  // SP2 phase initialisation
+  SP2_setPhase(SP2_PHASE_WAIT_CLIENTS);
+
   // manage server
   while(m_askThreadToEnd == false) {
     try {
@@ -322,6 +379,19 @@ int ServerThread::realThreadFunction() {
     }
   }
 
+  // close the server
+  close();
+
+  return 0;
+}
+
+int ServerThread::port() const {
+  return m_port;
+}
+
+void ServerThread::close() {
+  unsigned int i;
+
   // close the server
   m_acceptConnections = false;
 
@@ -352,7 +422,6 @@ int ServerThread::realThreadFunction() {
     StateManager::instance()->sendAsynchronousMessage("SERVER_STATUS_CHANGED");
   }
   LogInfo("server: ending normally");
-  return 0;
 }
 
 std::string ServerThread::SP2_determineLevel() {
@@ -364,6 +433,7 @@ std::string ServerThread::SP2_determineLevel() {
   v_result = m_pDb->readDB("SELECT id_level "
                            "FROM levels "
                            "WHERE isToReload=0 AND isScripted=0 AND isPhysics=0 " // warning, addforcetoplayer event cannot be over network game cause player id is serialized
+			   //"WHERE id_level='_iL00_' " // for tests
                            "ORDER BY RANDOM() LIMIT 1;",
 			 nrow);
   if(nrow == 0) {
@@ -381,24 +451,29 @@ void ServerThread::SP2_initPlaying() {
   unsigned int v_numPlayer;
   unsigned int v_localNetId = 0;
   std::string v_id_level;
-  m_fLastPhysTime = GameApp::getXMTime();
+  m_lastPhysTime = GameApp::getXMTimeInt();
 
   v_id_level = SP2_determineLevel();
 
   m_universe = new Universe();
   m_universe->initPlayServer();
 
+  // set hooks for the scenes
+  for(unsigned int i=0; i<m_universe->getScenes().size(); i++) {
+    m_universe->getScenes()[i]->setHooks(m_sceneHook);
+  }
+
   try {
     for(unsigned int i=0; i<m_universe->getScenes().size(); i++) {
 
       v_numPlayer = 0;
-      m_universe->getScenes()[i]->loadLevel(m_pDb, v_id_level);
+      m_universe->getScenes()[i]->loadLevel(m_pDb, v_id_level, true);
       if(m_universe->getScenes()[i]->getLevelSrc()->isXMotoTooOld()) {
 	throw Exception("Level " + v_id_level + " is too old");
       }
 
       m_DBuffer->clear();
-      m_universe->getScenes()[i]->prePlayLevel(m_DBuffer, true);
+      m_universe->getScenes()[i]->prePlayLevel(m_DBuffer, true, true, false);
       SP2_sendSceneEvents(m_DBuffer);
       
       // add the bikers
@@ -440,6 +515,12 @@ void ServerThread::SP2_initPlaying() {
 }
 
 void ServerThread::SP2_uninitPlaying() {
+  try {
+    m_rules->scriptCallVoid("Round_whenRound_ends");
+  } catch(Exception &e) {
+    // continue the game even if the rules are badly written
+    LogError(std::string("Rules: " + e.getMsg()).c_str());
+  }
   delete m_universe;
   m_universe = NULL;
 }
@@ -546,19 +627,32 @@ void ServerThread::SP2_updateScenePlaying() {
   bool v_firstFrame = false;
 
   if(SP2_managePreplayTime() == false) {
+
+    // manage the first time the update is done
+    if(m_sp2_gameStarted == false) {
+      m_sp2_gameStarted = true;
+
+      try {
+	m_rules->scriptCallVoid("Round_whenRound_begins");
+      } catch(Exception &e) {
+	// continue the game even if the rules are badly written
+	LogError(std::string("Rules: " + e.getMsg()).c_str());
+      }
+    }
+
     SP2_manageInactivity();
 
     /* update the scene */
     m_DBuffer->clear();
-    nPhysSteps =0;
+    nPhysSteps = 0;
 
-    while (m_fLastPhysTime + (PHYS_STEP_SIZE)/100.0 <= GameApp::getXMTime() && nPhysSteps < 10) {
+    while (m_lastPhysTime + (PHYS_STEP_SIZE*10) <= GameApp::getXMTimeInt() && nPhysSteps < 10) {
       for(unsigned int i=0; i<m_universe->getScenes().size(); i++) {
 	v_scene = m_universe->getScenes()[i];
-	v_scene->updateLevel(PHYS_STEP_SIZE, NULL, m_DBuffer, nPhysSteps!=0);
+	v_scene->updateLevel(PHYS_STEP_SIZE, NULL, m_DBuffer, nPhysSteps!=0, false /* no particles */, false /* don't update died players */);
       }
       v_updateDone = true;
-      m_fLastPhysTime += PHYS_STEP_SIZE/100.0;
+      m_lastPhysTime += PHYS_STEP_SIZE*10;
       nPhysSteps++;
     }
 
@@ -574,7 +668,7 @@ void ServerThread::SP2_updateScenePlaying() {
       v_firstFrame = true;
     }
     // initialize the physics time
-    m_fLastPhysTime = GameApp::getXMTime();
+    m_lastPhysTime = GameApp::getXMTimeInt();
   }
 
   // send to each client his frame and the frame of the others
@@ -632,16 +726,24 @@ void ServerThread::SP2_setPhase(ServerP2Phase i_sp2phase) {
 
   switch(m_sp2phase) {
   case SP2_PHASE_NONE:
-    m_wantedSleepingFramerate = 1;
     break;
 
   case SP2_PHASE_WAIT_CLIENTS:
-    m_wantedSleepingFramerate = 10;
+    // load rules if they changed
+    try {
+      if(m_needToReloadRules) {
+	m_needToReloadRules = false;
+	reloadRules(XM_SERVER_DEFAULT_RULES);
+      }
+    } catch(Exception &e) {
+      LogError((e.getMsg() + "\n" + m_rules->getErrorMsg()).c_str());
+    }
     break;
     
   case SP2_PHASE_PLAYING:
     m_firstFrameSent = GameApp::getXMTimeInt();
-    m_wantedSleepingFramerate = 200;
+    m_sp2_gameStarted = false;
+    m_sp2_lastLoopTime = -1;
     try {
       SP2_initPlaying();
     } catch(Exception &e) {
@@ -662,6 +764,9 @@ void ServerThread::SP2_unsetPhase() {
     
   case SP2_PHASE_PLAYING:
     SP2_uninitPlaying();
+
+    /* update scores */
+    sendPointsToSlavePlayers();
     break;
   }
 }
@@ -679,15 +784,10 @@ unsigned int ServerThread::nbClientsInMode(NetClientMode i_mode) {
 }
 
 void ServerThread::run_loop() {
-  if(manageNetwork()) {
-    m_nInactivNetLoop = 0; // reset net activity
-  } else {
-    m_nInactivNetLoop++;
-  }
-
   switch(m_sp2phase) {
 
   case SP2_PHASE_NONE:
+    manageNetwork(-1); // wait a network event
     break;
 
   case SP2_PHASE_WAIT_CLIENTS:
@@ -695,42 +795,64 @@ void ServerThread::run_loop() {
       // mark clients as to play
       for(unsigned int i=0; i<m_clients.size(); i++) {
 	if(m_clients[i]->mode() == NETCLIENT_SLAVE_MODE) {
-	  m_clients[i]->markToPlay(true);
+	  m_clients[i]->markToPlay(m_rules, true);
 	}
       }
       SP2_setPhase(SP2_PHASE_PLAYING);
+    } else {
+      manageNetwork(-1); // wait a network event
     }
     break;
     
   case SP2_PHASE_PLAYING:
+    // update the delta time to the reality
+    if(m_sp2_lastLoopTime == -1) {
+      m_sp2_lastLoopDelta = 0; // initialize the delta
+    } else {
+      m_sp2_lastLoopDelta += (GameApp::getXMTimeInt() - m_sp2_lastLoopTime) - 10; // update the delta
+    }
+    // too much delta, reset the delta
+    if(m_sp2_lastLoopDelta > 100 || m_sp2_lastLoopDelta < -100) {
+      m_sp2_lastLoopDelta = 0;
+    }
+    m_sp2_lastLoopTime = GameApp::getXMTimeInt();
+
     SP2_updateScenePlaying();
     SP2_updateCheckScenePlaying();
-    break;
-    
-  }
 
-  if(m_sp2phase == SP2_PHASE_PLAYING) {
-    /* that's not normal that if you comment the wait, it's not smooth, find why */
-    if(m_nInactivNetLoop > XM_SERVER_MIN_INACTIVITY_LOOP_TO_SLEEP) {
-      GameApp::wait(m_lastFrameTimeStamp, m_frameLate, m_wantedSleepingFramerate);
-    }
-  } else {
-    /* sleep only if network is not active */
-    if(m_nInactivNetLoop > XM_SERVER_MIN_INACTIVITY_LOOP_TO_SLEEP) {
-      SDL_Delay(XM_SERVER_UNPLAYING_SLEEP);
+    // mange the network according to time spent
+    // what is the remaing time on the 0.01s allowed
+    int v_remainingTime = 10 - (GameApp::getXMTimeInt() - m_sp2_lastLoopTime) - m_sp2_lastLoopDelta;
+    if(v_remainingTime > 0) {
+      manageNetwork(v_remainingTime);
     } else {
-      SDL_Delay(1); // always sleep the minimum time to limit the cpu usage
+      manageNetwork(0); // do at least one loop
     }
-  }
+    break;
 
+  }
 }
 
-bool ServerThread::manageNetwork() {
+/* 0 for no timeout, -1 for the maximum */
+void ServerThread::manageNetwork(int i_timeout) {
+  int v_start;
+  int v_timeout_remaining;
+
+  v_start             = GameApp::getXMTimeInt();
+  v_timeout_remaining = i_timeout;
+
+  do { // do at least one time, even for i_timeout = 0
+    manageNetworkOnePacket(v_timeout_remaining);
+    v_timeout_remaining = i_timeout - (GameApp::getXMTimeInt() - v_start /* = time spent */);
+  } while(v_timeout_remaining > 0);
+}
+
+bool ServerThread::manageNetworkOnePacket(int i_timeout) {
   int n_activ;
   unsigned int i;
   bool v_needMore = false;
 
-  n_activ = SDLNet_CheckSockets(m_set, 0);
+  n_activ = SDLNet_CheckSockets(m_set, i_timeout);
   if(n_activ == -1) {
     LogError("SDLNet_CheckSockets: %s", SDLNet_GetError());
     m_askThreadToEnd = true;
@@ -765,16 +887,18 @@ bool ServerThread::manageNetwork() {
 	      } else {
 		i++;
 	      }
-	    } catch(DisconnectedException &e) {
-	      LogInfo("server: client %u disconnected (%s:%d) : %s", i,
-		      XMNet::getIp(m_clients[i]->tcpRemoteIP()).c_str(),
-		      SDLNet_Read16(&(m_clients[i]->tcpRemoteIP())->port), e.getMsg().c_str());
-	      removeClient(i);
 	    } catch(Exception &e) {
-	      LogInfo("server: bad TCP packet received by client %u (%s:%d) : %s", i,
-		      XMNet::getIp(m_clients[i]->tcpRemoteIP()).c_str(),
-		      SDLNet_Read16(&(m_clients[i]->tcpRemoteIP())->port), e.getMsg().c_str());
-	      removeClient(i);
+	      if(e.getMsg() == "Disconnected") {// catch(DisconnectedException &e) won't work, i don't understand why
+		LogInfo("server: client %u disconnected (%s:%d) : %s", i,
+			XMNet::getIp(m_clients[i]->tcpRemoteIP()).c_str(),
+			SDLNet_Read16(&(m_clients[i]->tcpRemoteIP())->port), e.getMsg().c_str());
+		removeClient(i);
+	      } else {
+		LogInfo("server: bad TCP packet received by client %u (%s:%d) : %s", i,
+			XMNet::getIp(m_clients[i]->tcpRemoteIP()).c_str(),
+			SDLNet_Read16(&(m_clients[i]->tcpRemoteIP())->port), e.getMsg().c_str());
+		removeClient(i);
+	      }
 	    }
 	  } else {
 	    i++;
@@ -804,6 +928,9 @@ void ServerThread::removeClient(unsigned int i) {
     m_clients[i]->unbindUdp();
   }
 
+  // trigger the rules actions by removing the player of the sp2
+  m_clients[i]->markToPlay(m_rules, false);
+
   // send new client to other clients
   NA_changeClients nacc;
   NetInfosClient nic;
@@ -864,9 +991,13 @@ void ServerThread::sendToAllClientsHavingProtocol(int i_protocol, NetAction* i_n
     try {
       if((int)i != i_except) {
 	if(m_clients[i]->protocolVersion() < i_protocol) {
-	  sendToClient(i_netAction_lt, i, i_src, i_subsrc);
+	  if(i_netAction_lt != NULL) {
+	    sendToClient(i_netAction_lt, i, i_src, i_subsrc);
+	  }
 	} else {
-	  sendToClient(i_netAction_ge, i, i_src, i_subsrc);
+	  if(i_netAction_ge != NULL) {
+	    sendToClient(i_netAction_ge, i, i_src, i_subsrc);
+	  }
 	}
       }
     } catch(Exception &e) {
@@ -875,6 +1006,56 @@ void ServerThread::sendToAllClientsHavingProtocol(int i_protocol, NetAction* i_n
   }
 }
 
+void ServerThread::sendToAllClientsHavingModeAndProtocol(NetClientMode i_mode, int i_protocol, NetAction* i_netAction_lt, NetAction* i_netAction_ge, int i_src, int i_subsrc, int i_except) {
+  for(unsigned int i=0; i<m_clients.size(); i++) {
+    if((int)i != i_except && (i_mode == NETCLIENT_ANY_MODE || i_mode == m_clients[i]->mode())) {
+      try {
+	if((int)i != i_except) {
+	  if(m_clients[i]->protocolVersion() < i_protocol) {
+	    if(i_netAction_lt != NULL) {
+	      sendToClient(i_netAction_lt, i, i_src, i_subsrc);
+	    }
+	  } else {
+	    if(i_netAction_ge != NULL) {
+	      sendToClient(i_netAction_ge, i, i_src, i_subsrc);
+	    }
+	  }
+	}
+      } catch(Exception &e) {
+	// don't remove the client while removeclient function can call sendToAllClients ...
+      }
+    }
+  }
+}
+
+void ServerThread::sendPointsToSlavePlayers() {
+  for(unsigned int i=0; i<m_clients.size(); i++) {
+    if(m_clients[i]->mode() == NETCLIENT_SLAVE_MODE) {
+      sendPointsToClient(i);
+    }
+  }
+}
+
+void ServerThread::sendPointsToClient(unsigned int i_client) {
+  NA_slaveClientsPoints nascp;
+  NetPointsClient npc;
+  try {
+    for(unsigned int i=0; i<m_clients.size(); i++) {
+      if(m_clients[i]->mode() == NETCLIENT_SLAVE_MODE) {
+	if(i == i_client) {
+	  npc.NetId  = -1;
+	} else {
+	  npc.NetId  = m_clients[i]->id();
+	}
+	npc.Points = m_clients[i]->points();
+	nascp.add(&npc);
+      }
+    }
+    sendToClient(&nascp, i_client, -1, 0);
+  } catch(Exception &e) {
+  }
+}
+
 void ServerThread::acceptClient() {
   TCPsocket csd;
   IPaddress *tcpRemoteIP;
@@ -932,21 +1113,22 @@ void ServerThread::acceptClient() {
 }
 
 bool ServerThread::manageClientTCP(unsigned int i) {
-  NetAction* v_netAction;
-
-  while(m_clients[i]->tcpReader->TCPReadAction(m_clients[i]->tcpSocket(), &v_netAction)) {
-    if(manageAction(v_netAction, i) == false) {
-      delete v_netAction;
-      return false;
+  try {
+    while(m_clients[i]->tcpReader->TCPReadAction(m_clients[i]->tcpSocket(), &m_preAllocatedNA)) {
+      if(manageAction(m_preAllocatedNA.master, i) == false) {
+	return false;
+      }
     }
-    delete v_netAction;
+  } catch(Exception &e) {
+    m_unmanagedActions++;
+    throw e;
   }
+
   return true;
 }
 
 void ServerThread::manageClientUDP() {
   bool v_managedPacket;
-  NetAction* v_netAction;
 
   if(SDLNet_UDP_Recv(m_udpsd, m_udpPacket) == 1) {
     v_managedPacket = false;
@@ -955,14 +1137,14 @@ void ServerThread::manageClientUDP() {
 	 m_clients[i]->udpRemoteIP()->port == m_udpPacket->address.port)  {
 	v_managedPacket = true;
 	try {
-	  v_netAction = ActionReader::UDPReadAction(m_udpPacket->data, m_udpPacket->len);
-	  if(manageAction(v_netAction, i) == false) {
-	    delete v_netAction;
+	  ActionReader::UDPReadAction(m_udpPacket->data, m_udpPacket->len, &m_preAllocatedNA);
+	  if(manageAction(m_preAllocatedNA.master, i) == false) {
 	    removeClient(i);
 	    return;
 	  }
-	  delete v_netAction;
 	} catch(Exception &e) {
+	  m_unmanagedActions++;
+
 	  // ok, a bad packet received, forget it
 	  LogWarning("server: bad UDP packet received by client %u (%s:%i) : %s", i,
 		     XMNet::getIp(&(m_udpPacket->address)).c_str(), SDLNet_Read16(&(m_udpPacket->address.port)), e.getMsg().c_str());
@@ -974,11 +1156,11 @@ void ServerThread::manageClientUDP() {
     // anonym packet ? find the associated client
     if(v_managedPacket == false) {
       try {
-	v_netAction = ActionReader::UDPReadAction(m_udpPacket->data, m_udpPacket->len);
-	if(v_netAction->actionType() == TNA_udpBind) {
+	ActionReader::UDPReadAction(m_udpPacket->data, m_udpPacket->len, &m_preAllocatedNA);
+	if(m_preAllocatedNA.master->actionType() == TNA_udpBind) {
 	  for(unsigned int i=0; i<m_clients.size(); i++) {
 	    if(m_clients[i]->isUdpBinded() == false) {
-	      if(m_clients[i]->udpBindKey() == ((NA_udpBind*)v_netAction)->key()) {
+	      if(m_clients[i]->udpBindKey() == ((NA_udpBind*)m_preAllocatedNA.master)->key()) {
 		//LogInfo("UDP bind key received via UDP: %s", ((NA_udpBind*)v_netAction)->key().c_str());
 		m_clients[i]->bindUdp(m_udpPacket->address);
 		if(m_clients[i]->protocolVersion() >= 3) { // don't send if the version is lower because the client will not understand -- udp could not work in that case
@@ -1007,6 +1189,8 @@ void ServerThread::manageClientUDP() {
 	  LogWarning("Packet of unknown client received");
 	}
       } catch(Exception &e) {
+	m_unmanagedActions++;
+
 	/* forget this bad packet */
 	LogWarning("server: bad anonym UDP packet received by %s:%i",
 		   XMNet::getIp(&(m_udpPacket->address)).c_str(), SDLNet_Read16(&(m_udpPacket->address.port)));
@@ -1028,6 +1212,7 @@ bool ServerThread::manageAction(NetAction* i_netAction, unsigned int i_client) {
   case TNA_udpBindQuery:
   case TNA_serverError:
   case TNA_changeClients:
+  case TNA_slaveClientsPoints:
   case TNA_clientsNumber:
   case TNA_prepareToPlay:
   case TNA_prepareToGo:
@@ -1043,14 +1228,34 @@ bool ServerThread::manageAction(NetAction* i_netAction, unsigned int i_client) {
   case TNA_clientsNumberQuery:
     {
       // only clients having a name
-      int n=0;
+      int ntcp    = 0;
+      int nudp    = 0;
+      int nghosts = 0;
+      int nslaves = 0;
+
       for(unsigned int i=0; i<m_clients.size(); i++) {
 	if(m_clients[i]->name() != "") {
-	  n++;
+	  if(m_clients[i]->isUdpBindedValidated()) {
+	    nudp++;
+	  } else {
+	    ntcp++;
+	  }
+
+	  switch(m_clients[i]->mode()) {
+	  case NETCLIENT_GHOST_MODE:
+	    nghosts++;
+	    break;
+	  case NETCLIENT_SLAVE_MODE:
+	    nslaves++;
+	    break;
+	  case NETCLIENT_ANY_MODE:
+	    /* not a mode */
+	    break;
+	  }
 	}
       }
 
-      NA_clientsNumber nacn(n);
+      NA_clientsNumber nacn(ntcp, nudp, nghosts, nslaves);
       try {
 	sendToClient(&nacn, i_client, -1, 0);
       } catch(Exception &e) {
@@ -1090,7 +1295,7 @@ bool ServerThread::manageAction(NetAction* i_netAction, unsigned int i_client) {
       NetInfosClient nic;
       try {
 	for(unsigned int i=0; i<m_clients.size(); i++) {
-	  if(m_clients[i]->name() != "") {
+	  if(m_clients[i]->name() != "") { // remove still unset clients
 	    nic.NetId = m_clients[i]->id();
 	    nic.Name  = m_clients[i]->name();
 	    nacc.add(&nic);
@@ -1185,6 +1390,17 @@ bool ServerThread::manageAction(NetAction* i_netAction, unsigned int i_client) {
 	throw Exception("Invalid name provided");
       }
 
+      // length
+      if(utf8::utf8_length(m_clients[i_client]->name()) > XM_SERVER_NICK_LENGTH_MAX) {
+	NA_serverError na("Connexion refused : player name too long");
+	try {
+	  sendToClient(&na, i_client, -1, 0);
+	} catch(Exception &e) {
+	  /* ok, no pb */
+	}
+	return false;
+      }
+
       // check bans
       if(m_pDb->srv_isBanned(m_clients[i_client]->name(), XMNet::getIp(m_clients[i_client]->tcpRemoteIP()))) {
         LogInfo("server: banned client rejected (%s)", m_clients[i_client]->name().c_str());
@@ -1269,7 +1485,16 @@ bool ServerThread::manageAction(NetAction* i_netAction, unsigned int i_client) {
 	  return false;
 	}
       }
+
       m_clients[i_client]->setMode(((NA_clientMode*)i_netAction)->mode());
+
+      // require that the mode of the player is set (the line before)
+      if(((NA_clientMode*)i_netAction)->mode() == NETCLIENT_SLAVE_MODE) {
+	// send current scores to the player
+	if(m_clients[i_client]->protocolVersion() >= 5) {
+	  sendPointsToClient(i_client);
+	}
+      }
     }
     break;
 
@@ -1279,6 +1504,25 @@ bool ServerThread::manageAction(NetAction* i_netAction, unsigned int i_client) {
     }
     break;
 
+  case TNA_ping:
+    {
+      if(((NA_ping*)i_netAction)->isPong()) {
+	if(m_clients[i_client]->lastPing()->id == ((NA_ping*)i_netAction)->id()) {
+	  // same id, update the rcv time
+	  m_clients[i_client]->lastPing()->pongTime = GameApp::getXMTimeInt();
+	} else {
+	  // not the same last id, so, it means that an other ping has been send. Ignore this pong.
+	}
+      } else { // receive a ping
+	NA_ping na(((NA_ping*)i_netAction)); // send the pong as answer to the ping
+	try {
+	  sendToClient(&na, i_client, -1, 0);
+	} catch(Exception &e) {
+	}
+      }
+    }
+    break;
+
   }
 
   return true;
@@ -1304,7 +1548,7 @@ void ServerThread::manageSrvCmd(unsigned int i_client, const std::string& i_cmd)
       v_answer += "logout: disconnect from the server\n";
       v_answer += "changepassword <password>: change your password\n";
       v_answer += "lsplayers: list connected players\n";
-      v_answer += "lsxmversions: list xmoto versions used by players\n";
+      v_answer += "lsscores: give scores of players in slave mode\n";
       v_answer += "lsbans: list banned players\n";
 
       std::ostringstream v_n;
@@ -1314,6 +1558,8 @@ void ServerThread::manageSrvCmd(unsigned int i_client, const std::string& i_cmd)
       v_answer += "lsadmins: list admins\n";
       v_answer += "addadmin <id player> <password>: add player <id player> as admin\n";
       v_answer += "rmadmin <id admin>: remove admin\n";
+      v_answer += "reloadrules: reload server rules\n"; 
+      v_answer += "ping <all|id player>: information about player network connection to the server\n";
       v_answer += "stats: server statistics\n";
       v_answer += "msg <msg>: message to players\n";
     }
@@ -1349,7 +1595,9 @@ void ServerThread::manageSrvCmd(unsigned int i_client, const std::string& i_cmd)
     } else  if(m_clients[i_client]->isAdminConnected()) {
       v_answer += "Already connected\n";
     } else {
-      if(m_pDb->srv_isAdmin(m_clients[i_client]->name(), v_args[1])) {
+      if( (m_adminPassword != "" && m_adminPassword == std::string(v_args[1])) ||   // master admin password
+	  (m_pDb->srv_isAdmin(m_clients[i_client]->name(), v_args[1])) // normal admin password
+	  ) {
 	m_clients[i_client]->setAdminConnected(true);
 	v_answer += "Connected\n";
       } else {
@@ -1422,9 +1670,16 @@ void ServerThread::manageSrvCmd(unsigned int i_client, const std::string& i_cmd)
     if(v_args.size() != 1) {
       v_answer += "lsplayers: invalid arguments\n";
     } else {
-      char v_clientstr[48];
+      char v_clientstr[90];
       std::string v_mode;
+
+      v_answer += "+----+-----------------+----+----------------+---+------------------------+-----+--------+\n";
+      v_answer += "|  id|login            |mode|              ip|prt|version                 |xmprt|ping(mS)|\n";
+      v_answer += "+----+-----------------+----+----------------+---+------------------------+-----+--------+\n";
+
       for(unsigned int i=0; i<m_clients.size(); i++) {
+	std::ostringstream v_ping;
+
 	switch(m_clients[i]->mode()) {
 	case NETCLIENT_GHOST_MODE:
 	  v_mode = "GHOST";
@@ -1435,36 +1690,41 @@ void ServerThread::manageSrvCmd(unsigned int i_client, const std::string& i_cmd)
 	default:
 	  v_mode = "UNKWN";
 	}
+	if(m_clients[i]->lastPing()->pongTime == -1) {
+	  v_ping << "-";
+	} else {
+	  v_ping << (m_clients[i]->lastPing()->pongTime - m_clients[i]->lastPing()->pingTime);
+	}
 
-	snprintf(v_clientstr, 48, "%5u: %-12s %-5s %-21s",
+	snprintf(v_clientstr, 90, "%5u %-17s %-5s %15s %3s %-26s %3i %8s",
 		 m_clients[i]->id(), m_clients[i]->name().c_str(),
 		 v_mode.c_str(),
-		 ("(" + XMNet::getIp(m_clients[i]->tcpRemoteIP()) + "/" + 
-		  (m_clients[i]->isUdpBindedValidated() ? "UDP" : "TCP")
-		  + ")").c_str());
+		 (XMNet::getIp(m_clients[i]->tcpRemoteIP())).c_str(),
+		 m_clients[i]->isUdpBindedValidated() ? "UDP" : "TCP",
+		 m_clients[i]->xmversion().c_str(),
+		 m_clients[i]->protocolVersion(),
+		 v_ping.str().c_str()
+		 );
 	v_answer += v_clientstr;
 	v_answer += "\n";
       }
     }
 
-  } else if(v_args[0] == "lsxmversions") {
+  } else if(v_args[0] == "lsscores") {
     if(v_args.size() != 1) {
-      v_answer += "lsxmversions: invalid arguments\n";
+      v_answer += "lsscores: invalid arguments\n";
     } else {
-      char v_clientstr[55];
+      char v_clientstr[48];
+      std::string v_mode;
       for(unsigned int i=0; i<m_clients.size(); i++) {
-	snprintf(v_clientstr, 55, "%5u: %-12s (%-26s ; %3i)",
-		 m_clients[i]->id(), m_clients[i]->name().c_str(),
-		 m_clients[i]->xmversion().c_str(),
-		 m_clients[i]->protocolVersion());
-	v_answer += v_clientstr;
-	if(i % 3 == 2) {
+	if(m_clients[i]->mode() == NETCLIENT_SLAVE_MODE) {
+
+	  snprintf(v_clientstr, 48, "%5u: %-12s %6i",
+		   m_clients[i]->id(), m_clients[i]->name().c_str(), m_clients[i]->points());
+	  v_answer += v_clientstr;
 	  v_answer += "\n";
 	}
       }
-      if(m_clients.size() %3 != 2) {
-	v_answer += "\n";
-      }
     }
 
   } else if(v_args[0] == "lsbans") {
@@ -1517,13 +1777,75 @@ void ServerThread::manageSrvCmd(unsigned int i_client, const std::string& i_cmd)
       v_answer += "ban removed\n";
     }
 
+  } else if(v_args[0] == "reloadrules") {
+    if(v_args.size() != 1) {
+      v_answer += "reloadrules: invalid arguments\n";
+    } else {
+      m_needToReloadRules = true;
+      v_answer += "Rules will be reloaded just before the next round\n";
+    }
+
   } else if(v_args[0] == "stats") {
     if(v_args.size() != 1) {
       v_answer += "stats: invalid arguments\n";
     } else {
+      char v_line[256];
       v_answer += "start time : " + m_startTimeStr + "\n";
-      v_answer += NetAction::getStats() + "\n";
-      v_answer += ActionReader::getStats() + "\n";
+
+      //         | bytes | packets | biggest | 
+      // --------+-------+---------+---------|
+      //  in TCP |
+      //  in UDP |
+      // --------+
+      // out TCP |
+      // out UDP |
+
+      snprintf(v_line, 256, "+%9s+%13s+%13s+%13s+\n", "---------", "-------------", "-------------", "-------------");
+      v_answer += v_line;
+      snprintf(v_line, 256, "| %7s | %11s | %11s | %11s |\n", "       ", "   bytes   ", "  packets  ", "  biggest  ");
+      v_answer += v_line;
+      snprintf(v_line, 256, "+%9s+%13s+%13s+%13s+\n", "---------", "-------------", "-------------", "-------------");
+      v_answer += v_line;
+      snprintf(v_line, 256, "| %7s + %11s + %11i + %11s |\n", " in TCP",
+	       XMNet::getFancyBytes(ActionReader::m_TCPPacketsSizeReceived).c_str(),
+	       ActionReader::m_nbTCPPacketsReceived,
+	       XMNet::getFancyBytes(ActionReader::m_biggestTCPPacketReceived).c_str()
+	       );
+      v_answer += v_line;
+      snprintf(v_line, 256, "| %7s + %11s + %11i + %11s |\n", " in UDP",
+	       XMNet::getFancyBytes(ActionReader::m_UDPPacketsSizeReceived).c_str(),
+	       ActionReader::m_nbUDPPacketsReceived,
+	       XMNet::getFancyBytes(ActionReader::m_biggestUDPPacketReceived).c_str()
+	       );
+      v_answer += v_line;
+      snprintf(v_line, 256, "| %7s + %11s + %11i + %11s |\n", "out TCP",
+	       XMNet::getFancyBytes(NetAction::m_TCPPacketsSizeSent).c_str(),
+	       NetAction::m_nbTCPPacketsSent,
+	       XMNet::getFancyBytes(NetAction::m_biggestTCPPacketSent).c_str()
+	       );
+      v_answer += v_line;
+      snprintf(v_line, 256, "| %7s + %11s + %11i + %11s |\n", "out UDP",
+	       XMNet::getFancyBytes(NetAction::m_UDPPacketsSizeSent).c_str(),
+	       NetAction::m_nbUDPPacketsSent,
+	       XMNet::getFancyBytes(NetAction::m_biggestUDPPacketSent).c_str()
+	       );
+      v_answer += v_line;
+      snprintf(v_line, 256, "+%9s+%13s+%13s+%13s+\n", "---------", "-------------", "-------------", "-------------");
+      v_answer += v_line;
+
+      std::ostringstream v_nup;
+      v_answer += "unmanaged packets : ";
+      v_nup << m_unmanagedActions;
+      v_answer += v_nup.str() + "\n";
+
+      // db memory
+      std::ostringstream v_ndb;
+      v_answer += "db memory : ";
+      v_ndb << (xmDatabase::getMemoryUsed()) /1024/1024;
+      v_answer += v_ndb.str() + "mB" + "\n";
+
+      // system memory
+      v_answer += System::getMemoryInfo();
     }
 
   } else if(v_args[0] == "msg") {
@@ -1542,11 +1864,36 @@ void ServerThread::manageSrvCmd(unsigned int i_client, const std::string& i_cmd)
       NA_chatMessagePP na(v_msg.c_str(), "server", v_private_people);
       NA_chatMessage na_old(v_msg.c_str(), "server");
       try {
-	sendToAllClientsHavingProtocol(4, &na_old, &na, -1, 0, i_client);
+	sendToAllClientsHavingProtocol(4, &na_old, &na, -1, 0);
       } catch(Exception &e) {
       }
     }
 
+  } else if(v_args[0] == "ping") {
+    if(v_args.size() != 2) {
+      v_answer += "ping: invalid arguments\n";
+    } else {
+      NA_ping na;
+      int v_arg_client;
+
+      if(v_args[1] == "all") {
+	v_arg_client = -1;
+      } else {
+	v_arg_client = (int) atoi(v_args[1].c_str());
+      }
+
+      for(unsigned int i=0; i<m_clients.size(); i++) {
+	if(m_clients[i]->protocolVersion() >= 6) {
+	  if(m_clients[i]->id() == v_arg_client || v_arg_client == -1) {
+	    m_clients[i]->lastPing()->id = na.id();
+	    m_clients[i]->lastPing()->pingTime = GameApp::getXMTimeInt();
+	    m_clients[i]->lastPing()->pongTime = -1; // reset the pong time
+	    sendToClient(&na, i, -1, 0);
+	  }
+	}
+      }
+    }
+
   } else {
     v_answer = "Unknown command : \"" + v_args[0] + "\"\nType help to get more information\n";
   }
@@ -1556,10 +1903,20 @@ void ServerThread::manageSrvCmd(unsigned int i_client, const std::string& i_cmd)
   try {
     sendToClient(&na, i_client, -1, 0);
   } catch(Exception &e) {
-    /* ok, no pb */
+    /* ok, no pb retry with the error */
+    try {
+      NA_srvCmdAsw na("An error occured: " + e.getMsg());
+      sendToClient(&na, i_client, -1, 0);
+    } catch(Exception &e2) {
+      /* ok, no pb */
+    }
   }
 }
 
+NetSClient* ServerThread::getNetSClientById(unsigned int i_id) const {
+  return m_clients[getClientById(i_id)];
+}
+
 unsigned int ServerThread::getClientById(unsigned int i_id) const {
   for(unsigned int i=0; i<m_clients.size(); i++) {
     if(m_clients[i]->id() == i_id) {
@@ -1569,6 +1926,19 @@ unsigned int ServerThread::getClientById(unsigned int i_id) const {
   throw Exception("Client not found");
 }
 
+NetSClient* ServerThread::getNetSClientByScenePlayer(unsigned int i_numScene, unsigned int i_numPlayer) const {
+  for(unsigned int i=0; i<m_clients.size(); i++) {
+    if(m_clients[i]->isMarkedToPlay()) {
+      if(m_clients[i]->getNumScene() == i_numScene) {
+	if(m_clients[i]->getNumPlayer() == i_numPlayer) {
+	  return m_clients[i];
+	}
+      }
+    }
+  }
+  return NULL;
+}
+
 void ServerThread::cleanClientsMarkedToBeRemoved() {
 
   // main case : no client to remove
@@ -1592,17 +1962,6 @@ void ServerThread::cleanClientsMarkedToBeRemoved() {
   m_clientMarkToBeRemoved.clear();
 }
 
-void ServerThread::SP2_addPointsToClient(unsigned int i_client, unsigned int i_points) {
-//  NA_points na(m_clients[i_client]->id(), m_clients[i_client]->points());
-//
-//  try {
-//    sendToClient(&na, i_client, -1, 0);
-//    sendToAllClientsHavingMode(NETCLIENT_SLAVE_MODE, &na, m_clients[i]->id(), 0, i_client);
-//  } catch(Exception &e) {
-//    /* ok, no pb */
-//  }
-}
-
 bool ServerThread::acceptConnections() const {
   return m_acceptConnections;
 }
@@ -1624,3 +1983,105 @@ void ServerThread::sendMsgToClient(unsigned int i_client, const std::string& i_m
     }
   }
 }
+
+ServerRules* ServerThread::getRules() {
+  return m_rules;
+}
+
+Universe* ServerThread::getUniverse() {
+  return m_universe;
+}
+
+void ServerThread::reloadRules(const std::string& i_rulesFile) {
+  if(m_rules != NULL) {
+    delete m_rules;
+  }
+  m_rules = new ServerRules(this);
+
+  // init player points
+  for(unsigned int i=0; i<m_clients.size(); i++) {
+    m_clients[i]->setPoints(0);
+  }
+
+  m_rules->loadScriptFile(i_rulesFile);
+  m_rules->scriptCallVoid("Global_init");
+
+  // simulate the Global_whenPlayer_added for players as the rule ignore these players
+  for(unsigned int i=0; i<m_clients.size(); i++) {
+    if(m_clients[i]->mode() == NETCLIENT_SLAVE_MODE) {
+      m_rules->scriptCallVoidNumberArg("Global_whenPlayer_added", m_clients[i]->id());
+    }
+  }
+}
+
+XMServerSceneHooks::XMServerSceneHooks(ServerThread* i_st) {
+  m_server = i_st;
+}
+
+XMServerSceneHooks::~XMServerSceneHooks() {
+}
+
+void XMServerSceneHooks::OnEntityToTakeTakenByPlayer(unsigned int i_player) {
+  // only one scene is managed on the server -- for the moment
+  NetSClient* v_client = m_server->getNetSClientByScenePlayer(0, i_player);
+
+  if(v_client != NULL) {
+    try {
+      m_server->getRules()->scriptCallVoidNumberArg("Round_whenPlayer_onEntityToTakeTaken", v_client->id());
+    } catch(Exception &e) {
+      // continue the game even if the rules are badly written
+      LogError(std::string("Rules: " + e.getMsg()).c_str());
+    }
+  }
+}
+
+void XMServerSceneHooks::OnEntityToTakeTakenExternal() {
+  try {
+    m_server->getRules()->scriptCallVoid("Round_whenExternal_onEntityToTakeTaken");
+  } catch(Exception &e) {
+    // continue the game even if the rules are badly written
+    LogError(std::string("Rules: " + e.getMsg()).c_str());
+  }
+}
+
+void XMServerSceneHooks::OnPlayerWins(unsigned int i_player) {
+  // only one scene is managed on the server -- for the moment
+  NetSClient* v_client = m_server->getNetSClientByScenePlayer(0, i_player);
+
+  if(v_client != NULL) {
+    try {
+      m_server->getRules()->scriptCallVoidNumberArg("Round_whenPlayer_wins", v_client->id());
+    } catch(Exception &e) {
+      // continue the game even if the rules are badly written
+      LogError(std::string("Rules: " + e.getMsg()).c_str());
+    }
+  }
+}
+
+void XMServerSceneHooks::OnPlayerDies(unsigned int i_player) {
+  // only one scene is managed on the server -- for the moment
+  NetSClient* v_client = m_server->getNetSClientByScenePlayer(0, i_player);
+
+  if(v_client != NULL) {
+    try {
+      m_server->getRules()->scriptCallVoidNumberArg("Round_whenPlayer_dies", v_client->id());
+    } catch(Exception &e) {
+      // continue the game even if the rules are badly written
+      LogError(std::string("Rules: " + e.getMsg()).c_str());
+    }
+  }
+}
+
+void XMServerSceneHooks::OnPlayerSomersault(unsigned int i_player, bool i_counterclock) {
+  // only one scene is managed on the server -- for the moment
+  NetSClient* v_client = m_server->getNetSClientByScenePlayer(0, i_player);
+
+  if(v_client != NULL) {
+    try {
+      m_server->getRules()->scriptCallVoidNumberArg("Round_whenPlayer_DoesASomersault", v_client->id(), i_counterclock ? 1:0);
+    } catch(Exception &e) {
+      // continue the game even if the rules are badly written
+      LogError(std::string("Rules: " + e.getMsg()).c_str());
+    }
+  }
+}
