@@ -247,6 +247,8 @@ void StateManager::replaceState(GameState* pNewState, const std::string& i_paren
 }
 
 bool StateManager::needUpdateOrRender() {
+  int i;
+
   if(m_hasFocus) { // m_isVisible is not needed while xmoto is rendered after each event (including expose event)
     return true;
   }
@@ -255,7 +257,19 @@ bool StateManager::needUpdateOrRender() {
     return false;
   }
 
-  return m_statesStack[m_statesStack.size()-1]->updateWhenUnvisible();
+  // continue to find the upper state to update when unvisible
+  // or if the upper state is not to update when unvisible, perhaps
+  // its child is
+  i = m_statesStack.size()-1;
+  while(m_statesStack[i]->updateWhenUnvisible() == false &&
+	m_statesStack[i]->updateStatesBehind()) {
+    // down to the state 0
+    if(i == 0) {
+      return false; // no such state found
+    }
+    i--;
+  }
+  return m_statesStack[i]->updateWhenUnvisible();
 }
 
 void StateManager::update()
@@ -303,9 +317,6 @@ void StateManager::update()
     m_updateFpsNbFrame++;
   }
 
-  /* update net */
-  NetClient::instance()->executeNetActions(xmDatabase::instance("main"));
-
   /* update fps */
   if(m_lastFpsTime + 1000 < GameApp::getXMTimeInt()) {
     m_currentRenderFps = m_renderFpsNbFrame;
@@ -341,6 +352,8 @@ void StateManager::renderOverAll() {
 
     int nMX,nMY;
     GameApp::getMousePos(&nMX, &nMY);
+    int v_nameWidth;
+    int v_nameBorder = 10;
 
     // header
     v_fg = GameApp::instance()->getDrawLib()->getFontSmall()->getGlyph(GAMETEXT_CONNECTED_PLAYERS);
@@ -350,11 +363,24 @@ void StateManager::renderOverAll() {
     v_voffset += v_fg->realHeight();
     v_maxwidth = v_fg->realWidth();
 
-    // you
+    // you : name
     v_fg = GameApp::instance()->getDrawLib()->getFontSmall()->getGlyph(XMSession::instance()->profile());
     v_fm->printString(GameApp::instance()->getDrawLib(), v_fg,
 		      m_screen.getDispWidth() - v_fg->realWidth() - vborder, vborder+v_voffset,
-		      MAKE_COLOR(200,200,200,255), -1.0, true);       
+		      MAKE_COLOR(200,200,200,255), -1.0, true);
+    v_nameWidth = v_fg->realWidth();
+
+    // you : points
+    if(NetClient::instance()->mode() == NETCLIENT_SLAVE_MODE) {
+      std::ostringstream v_npoints;
+      v_npoints << "[ " << NetClient::instance()->points() << " ]";
+      v_fg = GameApp::instance()->getDrawLib()->getFontSmall()->getGlyph(v_npoints.str());
+      v_fm->printString(GameApp::instance()->getDrawLib(), v_fg,
+			m_screen.getDispWidth() - v_fg->realWidth() - vborder - v_nameWidth - v_nameBorder,
+			vborder+v_voffset,
+			MAKE_COLOR(255,150,150,255), -1.0, true);
+    }
+
     // update v_mouseOverPlayer
     if(nMX > m_screen.getDispWidth() - v_fg->realWidth() - vborder &&
        nMX < m_screen.getDispWidth()                     - vborder &&
@@ -372,9 +398,23 @@ void StateManager::renderOverAll() {
     for(unsigned int i=0; i<NetClient::instance()->otherClients().size(); i++) {
       v_fg = GameApp::instance()->getDrawLib()->getFontSmall()->getGlyph(NetClient::instance()->otherClients()[i]->name());
 
+      // others : name
       v_fm->printString(GameApp::instance()->getDrawLib(), v_fg,
 			m_screen.getDispWidth() - v_fg->realWidth() - vborder, vborder+v_voffset,
 			MAKE_COLOR(200,200,200,255), -1.0, true);     
+      v_nameWidth = v_fg->realWidth();
+
+      // others : points
+      if(NetClient::instance()->otherClients()[i]->mode() == NETCLIENT_SLAVE_MODE) {
+	std::ostringstream v_npoints;
+	v_npoints << "[ " << NetClient::instance()->otherClients()[i]->points() << " ]";
+	v_fg = GameApp::instance()->getDrawLib()->getFontSmall()->getGlyph(v_npoints.str());
+	v_fm->printString(GameApp::instance()->getDrawLib(), v_fg,
+			  m_screen.getDispWidth() - v_fg->realWidth() - vborder - v_nameWidth - v_nameBorder,
+			  vborder+v_voffset,
+			  MAKE_COLOR(255,150,150,255), -1.0, true);
+      }
+
       // update v_mouseOverPlayer
       if(nMX > m_screen.getDispWidth() - v_fg->realWidth() - vborder &&
 	 nMX < m_screen.getDispWidth()                     - vborder &&
@@ -512,6 +552,13 @@ void StateManager::render()
       drawGeomsLoading();
     }
 
+    // scraps
+    if(XMSession::instance()->debug()) {
+      // render scraps
+      FontManager* v_fm = drawLib->getFontSmall(); // any in fact while it's shared
+      v_fm->displayScrap(drawLib);
+    }
+
     // SYSMESSAGE
     SysMessage::instance()->render();
 
@@ -863,7 +910,7 @@ StateManager::registerAsEmitter(std::string message)
 }
 
 void
-StateManager::sendSynchronousMessage(std::string message, std::string args)
+StateManager::sendSynchronousMessage(std::string message, std::string args, const std::string& i_parentId)
 {
   std::map<std::string, std::vector<GameState*> >::iterator itFind;
 
@@ -880,10 +927,12 @@ StateManager::sendSynchronousMessage(std::string message, std::string args)
     }
 
     while(stateIterator != states.end()){
-      (*stateIterator)->executeOneCommand(message, args);
+      if(i_parentId == "" || (*stateIterator)->getStateId() == i_parentId) {
+	(*stateIterator)->executeOneCommand(message, args);
 
-      LogDebug("sendSynchronousMessage [%s [%s]] to [%s]",
-	       message.c_str(), args.c_str(), (*stateIterator)->getName().c_str());
+	LogDebug("sendSynchronousMessage [%s [%s]] to [%s] /* parent_id = '%s' */",
+		 message.c_str(), args.c_str(), (*stateIterator)->getName().c_str(), i_parentId.c_str());
+      }
 
       ++stateIterator;
     }
@@ -896,7 +945,7 @@ StateManager::sendSynchronousMessage(std::string message, std::string args)
 }
 
 void
-StateManager::sendAsynchronousMessage(std::string message, std::string args)
+StateManager::sendAsynchronousMessage(std::string message, std::string args, const std::string& i_parentId)
 {
   std::map<std::string, std::vector<GameState*> >::iterator itFind;
 
@@ -913,11 +962,15 @@ StateManager::sendAsynchronousMessage(std::string message, std::string args)
     }
 
     while(stateIterator != states.end()){
-      LogDebug("sendAsynchronousMessage [%s [%s]] to [%s]",
-	       message.c_str(), args.c_str(),
-	       (*stateIterator)->getName().c_str());
-
-      (*stateIterator)->send(message, args);
+ 
+      if(i_parentId == "" || (*stateIterator)->getStateId() == i_parentId) {
+	LogDebug("sendAsynchronousMessage [%s [%s]] to [%s] /* parent_id = '%s' */",
+		 message.c_str(), args.c_str(),
+		 (*stateIterator)->getName().c_str(),
+		 i_parentId.c_str());
+	
+	(*stateIterator)->send(message, args);
+      }
       ++stateIterator;
     }
   } else {
