@@ -24,11 +24,11 @@ Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 #include <string>
 #include "../helpers/Color.h"
 #include "../helpers/VMath.h"
+#include "../VXml.h"
 #include "BasicSceneStructs.h"
 #include <vector>
 
 class EntityParticle;
-class TiXmlElement;
 class FileHandle;
 class Sprite;
 class Block;
@@ -76,9 +76,6 @@ class Entity {
     return m_sprite;
   }
   bool  isAlive() const;
-  bool  isScripted() const {
-    return m_isScripted;
-  }
 
   virtual void loadToPlay(const std::string& i_ScriptSource);
   virtual void unloadToPlay();
@@ -107,7 +104,7 @@ class Entity {
   void setAlive(bool alive);
 
   void saveBinary(FileHandle *i_pfh);
-  static Entity* readFromXml(TiXmlElement *pElem);
+  static Entity* readFromXml(xmlNodePtr pElem);
   static Entity* readFromBinary(FileHandle *i_pfh);
 
   static EntitySpeciality  SpecialityFromStr(std::string& i_typeStr);
@@ -118,7 +115,7 @@ class Entity {
   }
   void setSpeciality(EntitySpeciality i_speciality);
 
-  virtual bool updateToTime(int i_time, Vector2f& i_gravity, PhysicsSettings* i_physicsSettings);
+  virtual bool updateToTime(int i_time, Vector2f& i_gravity, PhysicsSettings* i_physicsSettings, bool i_allowParticules);
 
   AABB& getAABB();
 
@@ -141,7 +138,6 @@ protected:
   bool        m_isCheckpoint;
   /* Use to know if a script shall update the pos of the entity*/
   bool        m_isAlive;
-  bool        m_isScripted;   /* is it used in a script? */
   TColor      m_color;
 
   BoundingCircle m_BCircle;
@@ -225,7 +221,7 @@ public:
 
   void saveBinary(FileHandle *i_pfh);
   void readFromBinary(FileHandle *i_pfh);
-  void readFromXml(TiXmlElement *pElem);
+  void readFromXml(xmlNodePtr pElem);
 
   void loadToPlay(Level* i_level, ChipmunkWorld* i_chipmunkWorld);
   void unloadToPlay();
@@ -257,7 +253,7 @@ class ParticlesSource : public Entity {
 
   virtual void loadToPlay();
   virtual void unloadToPlay();
-  virtual bool updateToTime(int i_time, Vector2f& i_gravity, PhysicsSettings* i_physicsSettings);
+  virtual bool updateToTime(int i_time, Vector2f& i_gravity, PhysicsSettings* i_physicsSettings, bool i_allowParticules);
   inline std::vector<EntityParticle*>& Particles() {
     return m_particles;
   }
@@ -313,7 +309,7 @@ class ParticlesSourceSmoke : public ParticlesSourceMultiple {
   ParticlesSourceSmoke(const std::string& i_id);
   virtual ~ParticlesSourceSmoke();
 
-  bool updateToTime(int i_time, Vector2f& i_gravity, PhysicsSettings* i_physicsSettings);
+  bool updateToTime(int i_time, Vector2f& i_gravity, PhysicsSettings* i_physicsSettings, bool i_allowParticules);
   void addParticle(int i_curTime);
 };
 
@@ -324,7 +320,7 @@ class ParticlesSourceFire : public ParticlesSource {
   ParticlesSourceFire(const std::string& i_id);
   virtual ~ParticlesSourceFire();
 
-  bool updateToTime(int i_time, Vector2f& i_gravity, PhysicsSettings* i_physicsSettings);
+  bool updateToTime(int i_time, Vector2f& i_gravity, PhysicsSettings* i_physicsSettings, bool i_allowParticules);
   void addParticle(int i_curTime);
 };
 
@@ -341,7 +337,7 @@ class ParticlesSourceDebris : public ParticlesSource {
   ParticlesSourceDebris(const std::string& i_id);
   virtual ~ParticlesSourceDebris();
 
-  bool updateToTime(int i_time, Vector2f& i_gravity, PhysicsSettings* i_physicsSettings);
+  bool updateToTime(int i_time, Vector2f& i_gravity, PhysicsSettings* i_physicsSettings, bool i_allowParticules);
   void addParticle(int i_curTime);
 };
 
@@ -350,7 +346,7 @@ class ParticlesSourceSparkle : public ParticlesSource {
   ParticlesSourceSparkle(const std::string& i_id);
   virtual ~ParticlesSourceSparkle();
 	int m_last_time;
-  bool updateToTime(int i_time, Vector2f& i_gravity, PhysicsSettings* i_physicsSettings);
+  bool updateToTime(int i_time, Vector2f& i_gravity, PhysicsSettings* i_physicsSettings, bool i_allowParticules);
   void addParticle(int i_curTime);
 };
 
@@ -363,7 +359,7 @@ class EntityParticle : public Entity {
   EntityParticle();
   virtual ~EntityParticle();
 
-  virtual  bool updateToTime(int i_time, Vector2f& i_gravity, PhysicsSettings* i_physicsSettings);
+  virtual  bool updateToTime(int i_time, Vector2f& i_gravity, PhysicsSettings* i_physicsSettings, bool i_allowParticules);
   inline   int  KillTime() const {
     return m_killTime;
   }
@@ -397,7 +393,7 @@ class SmokeParticle : public EntityParticle {
   virtual ~SmokeParticle();
   void init(const Vector2f& i_position, const Vector2f& i_velocity, int i_killTime, std::string i_spriteName);
 
-  virtual bool updateToTime(int i_time, Vector2f& i_gravity, PhysicsSettings* i_physicsSettings);
+  virtual bool updateToTime(int i_time, Vector2f& i_gravity, PhysicsSettings* i_physicsSettings, bool i_allowParticules);
 
  private:
 };
@@ -408,7 +404,7 @@ class FireParticle : public EntityParticle {
   virtual ~FireParticle();
   void init(const Vector2f& i_position, const Vector2f& i_velocity, int i_killTime, std::string i_spriteName);
 
-  virtual bool updateToTime(int i_time, Vector2f& i_gravity, PhysicsSettings* i_physicsSettings);
+  virtual bool updateToTime(int i_time, Vector2f& i_gravity, PhysicsSettings* i_physicsSettings, bool i_allowParticules);
 
  private:
   float m_fireSeed;
@@ -429,7 +425,7 @@ class DebrisParticle : public EntityParticle {
   virtual ~DebrisParticle();
   void init(const Vector2f& i_position, const Vector2f& i_velocity, int i_killTime, std::string i_spriteName);
 
-  virtual bool updateToTime(int i_time, Vector2f& i_gravity, PhysicsSettings* i_physicsSettings);
+  virtual bool updateToTime(int i_time, Vector2f& i_gravity, PhysicsSettings* i_physicsSettings, bool i_allowParticules);
 
  private:
 };
@@ -440,7 +436,7 @@ class SparkleParticle : public EntityParticle {
   virtual ~SparkleParticle();
   void init(const Vector2f& i_position, const Vector2f& i_velocity, int i_killTime, std::string i_spriteName);
 
-  virtual bool updateToTime(int i_time, Vector2f& i_gravity, PhysicsSettings* i_physicsSettings);
+  virtual bool updateToTime(int i_time, Vector2f& i_gravity, PhysicsSettings* i_physicsSettings, bool i_allowParticules);
 
  private:
   float m_SparkleSeed;
